/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/

.globl stack_start
.globl start
.globl __bss_start
.globl __bss_end
.set noreorder
.set noat


#include "../hypercall.h"

.globl entry
.section .vecs
entry:
	b     initial_entry
	nop
	
promcall:
	b     handle_promcall
	nop

initial_entry:
#ifdef SELF_LOAD
	//we are either calling ourselves the first time or second, first time will be at the wrong
	// address (0x80000000) and require PIC technique and fitting into one sector
	//load first 32 sectors to 0x80001000, then jump to real_start
	li    $s0, 0x00001000
	li    $s1, 0
	li    $s2, 32
1:
	move  $a0, $s1
	bal   readblock
	move  $a1, $s0
	addiu $s1, 1
	bne   $s1, $s2, 1b
	addiu $s0, 512
	
	la    $s0, real_start
	jr    $s0
	nop

#endif

.globl real_start
real_start:
	la    $sp, stack_start	//setup SP
	mfc0  $t0, $12		//12 = status
	ori   $t0, $t0, 3	//ints off, kernel mode
	xori  $t0, $t0, 3
	mtc0  $t0, $12		//12 = status
	la    $t0, __bss_start
	la    $t1, __bss_end
1:
	beq   $t0, $t1, start
	addiu $t0, 4
	bal   1b
	sw    $zero, -4($t0)

//also in .vec to fit into ferst sector in case of "SELF_LOAD"
.globl readblock
readblock:		//(a0 = block number, a1 = destination PA)
	li    $at, H_STOR_READ
	jr    $ra
	.word HYPERCALL


.section .text

dispatch_promcall:
	addu  $v1, $ra
	lw    $v1, ($v1)
	jr    $v1
	move  $ra, $at

.globl handle_promcall
handle_promcall:
	move  $at, $ra
	bal   dispatch_promcall
	nop
	.word 0xbfcfffff	//0
	.word 0xbfcfffff	//1 = exec
	.word prom_restart_reboot_halt
	.word 0xbfcfffff	//3
	.word prom_restart_reboot_halt
	.word 0xbfc00000	//reboot automatically to same thing we just did
	.word 0xbfcfffff	//6 = open
	.word 0xbfcfffff	//7 = read
	.word 0xbfcfffff	//8 = write
	.word 0xbfcfffff	//9
	.word 0xbfcfffff	//10 = close
	.word 0xbfcfffff	//11 = lseek
	.word prom_getchar
	.word prom_putchar
	.word 0xbfcfffff	//14
	.word 0xbfcfffff	//15 = gets
	.word 0xbfcfffff	//16
	.word pr
	.word 0xbfcfffff	//18
	.word 0xbfcfffff	//19
	.word 0xbfcfffff	//20
	.word 0xbfcfffff	//21
	.word 0xbfcfffff	//22
	.word 0xbfcfffff	//23
	.word 0xbfcfffff	//24
	.word 0xbfcfffff	//25
	.word 0xbfcfffff	//26
	.word 0xbfcfffff	//27
	.word 0xbfcfffff	//28
	.word 0xbfcfffff	//29
	.word 0xbfcfffff	//30
	.word 0xbfcfffff	//31
	.word 0xbfcfffff	//32
	.word prom_getenv	//33

.globl getStoreSz
getStoreSz:
	li    $at, H_STOR_GET_SZ
	jr    $ra
	.word HYPERCALL

.globl getMemMap
getMemMap:
	li    $at, H_GET_MEM_MAP
	jr    $ra
	.word HYPERCALL

.globl consoleWrite
consoleWrite:
	li    $at, H_CONSOLE_WRITE
	jr    $ra
	.word HYPERCALL

.globl writeblock
writeblock:		//(a0 = block number, a1 = source PA)
	li    $at, H_STOR_WRITE
	jr    $ra
	.word HYPERCALL
	



.balign 32

.globl memcpy
memcpy:		//(a0 = dst, a1 = src, a2 = len)
	beqz  $a2, 1f
	move  $v0, $a0
	
	or    $v1, $a0, $a1
	andi  $v1, $v1, 3
	bnez  $v1, byteloop

	srl   $v1, $a2, 2
	andi  $a2, $a2, 3

.balign 32	//should be a single nop

wordloop:
	beqz  $v1, byteloop
	addiu $v1, $v1, -1
	lw    $a3, 0($a1)
	sw    $a3, 0($a0)
	addiu $a1, $a1, 4
	b     wordloop
	addiu $a0, $a0, 4

byteloop:
	beqz  $a2, 1f
	addiu $a2, $a2, -1
	lbu   $a3, 0($a1)
	sb    $a3, 0($a0)
	addiu $a1, $a1, 1
	b     byteloop
	addiu $a0, $a0, 1
1:
	jr    $ra
	nop

.globl strcmp
strcmp:		//(a0 = str1, a1 = str2)
	lbu   $a2, 0($a0)
	beqz  $a2, 1f
	lbu   $a3, 0($a1)
	beqz  $a3, 1f
	addiu $a0, $a0, 1
	bne   $a2, $a3, 1f
	addiu $a1, $a1, 1
	b     strcmp
	nop
	
1:
	jr    $ra
	sub   $v0, $a2, $a3

.globl memset
memset:		//a0 = dst, a1 = byte, a2 = len
	move  $v0, $a0
2:
	beqz  $a2, 1f
	addiu $a2, $a2, -1
	sb    $a1, 0($a0)
	b     2b
	addiu $a0, $a0, 1
1:
	jr    $ra
	nop


.globl callWithGpVal
callWithGpVal:
	//ultrix insists args be in kseg1, linux does not care. we are in kseg0, so add 0x20000000
	.set noreorder
	li   $at, 0xa0000000 - 0x80000000
	addu $a1, $at
	lw   $v0, 0x14($sp)
	jr   $v0
	lw   $gp, 0x10($sp)
