/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/

#include "partitions.h"
#include "printf.h"
#include "util.h"


#define DISKLABEL_MAGIC					0x82564557
#define NETBSD_MAX_PARTITIONS			22

//see man 5 disklabel
struct NetBSDpartition {
	uint32_t numSec;
	uint32_t firstSec;
	uint32_t fsFragSz;
	uint8_t fsType;
	uint8_t fsFragsPerBlock;
	union {
		uint16_t cpg;	//ufs
		uint16_t sgs;	//lfs
	};
};	//0x10 in size

struct NetBDSdisklabel {
	uint32_t bootcode[16];				//0x00
	uint32_t magic;						//0x40
	uint16_t driveType;					//0x44
	uint16_t driveSubtype;				//0x46
	char driveTypeName[16];				//0x48
	char drivePackName[16];				//0x58
	uint32_t bytesPerSec;				//0x68
	uint32_t secPerTrack;				//0x6c
	uint32_t tracksPerCyl;				//0x70
	uint32_t cylsPerUnit;				//0x74
	uint32_t secsPerCyl;				//0x78
	uint32_t secsPerUnit;				//0x7c
	uint16_t spareSecsPerPerTrack;		//0x80
	uint16_t spareSecsPerPerCyl;		//0x82
	uint32_t altCylsPerUnit;			//0x84
	uint32_t misc[15];					//0x88
	uint32_t magic2;					//0xc4
	uint16_t checksum;					//0xc8
	uint16_t nPartitions;				//0xca	remember: NETBSD_MAX_PARTITIONS
	uint32_t bootAreaSz;				//0xcc	in bytes
	uint32_t fsSuperblockSz;			//0xd0	in bytes
	struct NetBSDpartition part[NETBSD_MAX_PARTITIONS];	//0xd4
};

#define FS_TYPE_SWAP					1
#define FS_TYPE_BSD42					7


bool partMgrFindBootPartition(PartMgrDiskBlockRead readF, void *readD, void *buf512, uint32_t *startSecP, uint32_t *numSecP)
{
	struct NetBDSdisklabel *dl = (struct NetBDSdisklabel*)buf512;
	
	if (!readF(readD, dl, 0))
		fatal("Failed to read disklabel\n");
	else if (dl->magic != DISKLABEL_MAGIC || dl->magic2 != DISKLABEL_MAGIC || !dl->nPartitions || dl->nPartitions > NETBSD_MAX_PARTITIONS)
		fatal("Disklabel is invalid\n");
	else if (dl->part[0].fsType != FS_TYPE_BSD42)
		fatal("Part type is wrong\n");
	else {
		
		if (startSecP)
			*startSecP = dl->part[0].firstSec;
		if (numSecP)
			*numSecP = dl->part[0].numSec;
		
		return true;
	}
	return false;
}


