/*
	(c) 2021 Dmitry Grinberg   https://dmitry.gr
	Non-commercial use only OR licensing@dmitry.gr
*/

#ifndef _UFS_H_
#define _UFS_H_

#include <stdbool.h>
#include <stdint.h>

#define UFS_SECTOR_SIZE		(512)




typedef bool (*UfsReadSecProc)(void *userData, uint32_t sec, void *dst);

struct UfsFileOpenInfoOpaque {
	uint32_t tbd;
};

#define UFS_MAX_NAME_LEN			255

struct UfsVolume;
struct UfsFile;

void ufsInit(void *tmpBuf);		//512 bytes

struct UfsVolume* ufsMount(UfsReadSecProc readSecF, void *readSecD);
void ufsUnmount(struct UfsVolume *vol);

struct UfsFile* ufsOpenRoot(struct UfsVolume *vol);
struct UfsFile* ufsFileOpen(struct UfsVolume* vol, struct UfsFileOpenInfoOpaque *oiP);
void ufsFileClose(struct UfsFile *file);

bool ufsDirEnum(struct UfsFile *dir, char *nameP, struct UfsFileOpenInfoOpaque *oiP);
bool ufsFileInfo(struct UfsVolume* vol, struct UfsFileOpenInfoOpaque *oiP, uint16_t *modeP, uint8_t *typeP, uint32_t *sizeP);

uint32_t ufsFileRead(struct UfsFile *file, void *buf, uint32_t len);
uint32_t ufsFileSkip(struct UfsFile *file, uint32_t len);	//seek forwards
uint32_t ufsFileTell(const struct UfsFile *file);
bool ufsFileSeek(struct UfsFile *file, uint32_t pos);
void ufsFileRewind(struct UfsFile *file);


#endif
